﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WarmlyLocomotive
{
    internal class SetLocomotivesGeneric<T>
        where T : class
    {
        /// <summary>
		/// Массив объектов, которые храним
		/// </summary>
		private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetLocomotivesGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="locomotive">Добавляемый локомотив</param>
        /// <returns></returns>
        public int Insert(T locomotive)
        {
            bool hasEmptySpace = false;
            int indexOfEmptyPlace = 0;
            for (int i = 0; i < Count; ++i)
            {
                if (_places[i] == null || _places[i] == default(T))
                {
                    hasEmptySpace = true;
                    indexOfEmptyPlace = i;
                    break;
                }
            }
            if (hasEmptySpace)
            {
                for (int i = indexOfEmptyPlace; i > 0; --i)
                {
                    _places[i] = _places[i - 1];
                }
                _places[0] = locomotive;
                return 0;
            }
            else
            {
                return -1;
            }
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="locomotive">Добавляемый локомотив</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T locomotive, int position)
        {
            if (position < 0 || position >= Count)
            {
                return -1;
            }
            if (_places[position] == null || _places[position] == default(T))
            {
                _places[position] = locomotive;
                return position;
            }
            else
            {
                bool hasEmptySpace = false;
                int indexOfEmptyPlace = 0;
                for (int i = position + 1; i < Count; ++i)
                {
                    if (_places[i] == null || _places[i] == default(T))
                    {
                        hasEmptySpace = true;
                        indexOfEmptyPlace = i;
                        break;
                    }
                }
                if (hasEmptySpace)
                {
                    for (int i = indexOfEmptyPlace; i > position; --i)
                    {
                        _places[i] = _places[i - 1];
                    }
                    _places[position] = locomotive;
                    return position;
                }
                else
                {
                    return -1;
                }
            }
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= Count)
            {
                return null;
            }
            T memoryObj = _places[position];
            _places[position] = null;
            return memoryObj;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Get(int position)
        {
            if (position < 0 || position >= Count)
            {
                return null;
            }
            return _places[position];
        }
    }
}
